==> java-module-2.9.3-20250423-sources.jar: org/mule/weave/v2/module/java/JavaFieldRefFunction.scala <==
package org.mule.weave.v2.module.java

import org.mule.weave.v2.core.functions.SecureBinaryFunctionValue
import org.mule.weave.v2.model.EvaluationContext
import org.mule.weave.v2.model.capabilities.UnknownLocationCapable
import org.mule.weave.v2.model.service.WeaveRuntimePrivilege
import org.mule.weave.v2.model.structure.KeyValuePair
import org.mule.weave.v2.model.types.ArrayType
import org.mule.weave.v2.model.types.ObjectType
import org.mule.weave.v2.model.types.StringType
import org.mule.weave.v2.model.values.ArrayValue
import org.mule.weave.v2.model.values.ObjectValue
import org.mule.weave.v2.model.values.Value
import org.mule.weave.v2.model.values.wrappers.WrapperValue
import org.mule.weave.v2.module.commons.java.value.JavaSchema
import org.mule.weave.v2.module.commons.java.value.JavaValue
import org.mule.weave.v2.module.commons.java.writer.JavaAdapter
import org.mule.weave.v2.module.pojo.ClassLoaderService
import org.mule.weave.v2.module.pojo.DefaultClassLoaderService
import org.mule.weave.v2.module.pojo.JavaDataFormat
import org.mule.weave.v2.module.pojo.exception.IllegalReadPropertyAccessException
import org.mule.weave.v2.module.pojo.exception.JavaPropertyAccessException
import org.mule.weave.v2.module.pojo.reader.ReflectionJavaValueConverter
import org.mule.weave.v2.module.writer.Writer
import org.mule.weave.v2.parser.exception.LocatableException

import java.lang.reflect.Field

class JavaFieldRefFunction extends SecureBinaryFunctionValue {

  override val L = StringType

  override val R = StringType

  override val requiredPrivilege: WeaveRuntimePrivilege = WeaveRuntimePrivilege.JAVA_REFLECTION

  override def onSecureExecution(classNameValue: L.V, fieldNameValue: R.V)(implicit ctx: EvaluationContext): Value[_] = {
    val className: String = classNameValue.evaluate.toString
    val fieldName: String = fieldNameValue.evaluate.toString
    try {
      val classToCall = JavaInvocationHelper.loadClass(className)
      val field: Field = classToCall.getDeclaredField(fieldName)
      if (!ctx.serviceManager.settingsService.java().disableSetAccessible) {
        field.setAccessible(true)
      }
      ReflectionJavaValueConverter.convert(field.get(null), () => className + "." + fieldName)
    } catch {
      case le: LocatableException =>
        throw le
      case e @ (_: IllegalAccessException | _: IllegalArgumentException) =>
        throw new IllegalReadPropertyAccessException(fieldName, className, this, e.getCause)
      case e: Exception =>
        throw new JavaPropertyAccessException(fieldName, className, fieldNameValue, e)
    }
  }
}

object JavaFieldRefFunction {

  def transformArgumentsToJava(ctx: EvaluationContext, paramTypes: Seq[Class[_]], argsSeq: Seq[Value[_]]): Seq[AnyRef] = {
    argsSeq
      .zip(paramTypes)
      .map((value) => {
        transformToJava(value._1, value._2)(ctx)
      })
  }

  def transformToJavaMap(arg: Value[_], mapValueClass: Class[_], mapClass: Class[_])(implicit ctx: EvaluationContext): AnyRef = {
    val writer: Writer = JavaDataFormat.defaultSettingsWriter(None)
    val map = ObjectType
      .coerce(arg)
      .evaluate
      .toIterator()
      .map((kvp) => {
        KeyValuePair(kvp._1, adaptToJavaClass(kvp._2, mapValueClass))
      })
    val valueToWrite = adaptToJavaClass(ObjectValue(map, UnknownLocationCapable), mapClass)
    writer.writeValue(valueToWrite)(ctx)
    writer.result.asInstanceOf[AnyRef]
  }

  def transformToJavaCollection(argValue: Value[_], item: Class[_], collectionClass: Class[_])(implicit ctx: EvaluationContext): AnyRef = {
    val writer: Writer = JavaDataFormat.writer(None)
    val values: Iterator[Value[_]] = ArrayType
      .coerce(argValue)
      .evaluate
      .toIterator()
      .map((value) => {
        adaptToJavaClass(value, item)
      })
    val valueToWrite = adaptToJavaClass(ArrayValue(values, UnknownLocationCapable), collectionClass)
    writer.writeValue(valueToWrite)(ctx)
    writer.result.asInstanceOf[AnyRef]
  }

  def transformToJava(argValue: Value[_], expectedJavaType: Class[_])(implicit ctx: EvaluationContext): AnyRef = {
    val maybeAdapter = ValueToJavaAdaptorManager.findAdapterForClass(expectedJavaType)
    maybeAdapter match {
      case Some(valueAdapter) => {
        valueAdapter.adaptToJavaValue(argValue, expectedJavaType, ctx)
      }
      case None => {
        val writer: Writer = JavaDataFormat.defaultSettingsWriter(None)
        val valueToWrite: Value[_] = adaptToJavaClass(argValue, expectedJavaType)
        writer.writeValue(valueToWrite)(ctx)
        JavaAdapter.fromScalaToJava(writer.result).asInstanceOf[AnyRef]
      }
    }
  }

  def adaptToJavaClass(argValue: Value[_], expectedJavaType: Class[_])(implicit ctx: EvaluationContext): Value[_] = {
    //Should we inferred the class?
    val hasClassDefined: Boolean = doesHasClassDefined(argValue, expectedJavaType)
    //If it says object don't use it
    val valueToWrite = if (!hasClassDefined && !classOf[Object].equals(expectedJavaType)) {
      val typeCoercer = argValue.valueType(ctx).baseType.coercer()
      argValue.valueType(ctx).baseType.withSchema(Some(JavaSchema(expectedJavaType))).coerce(argValue)(ctx)
    } else {
      argValue
    }
    valueToWrite
  }

  private def doesHasClassDefined(argValue: Value[_], expectedJavaType: Class[_])(implicit ctx: EvaluationContext): Boolean = {
    argValue.schema match {
      case Some(schema) => {
        argValue match {
          case jv: JavaValue[_] => {
            val underlying = jv.underlying()
            expectedJavaType.isInstance(underlying)
          }
          case wv: WrapperValue => doesHasClassDefined(wv.value, expectedJavaType)
          case _ => {
            val maybeString = schema.`class`
            maybeString match {
              case None => false
              case Some(value) => {
                val classLoaderService = ctx.serviceManager.lookupCustomService(classOf[ClassLoaderService], DefaultClassLoaderService)
                classLoaderService.loadClass(value) match {
                  case None              => false
                  case Some(loadedClass) => expectedJavaType.isAssignableFrom(loadedClass)
                }
              }
            }
          }
        }
      }
      case _ => false
    }
  }
}

==> java-module-2.10.0-20250523.192847-113-sources.jar: org/mule/weave/v2/module/java/JavaFieldRefFunction.scala <==
package org.mule.weave.v2.module.java

import org.mule.weave.v2.core.functions.SecureBinaryFunctionValue
import org.mule.weave.v2.model.EvaluationContext
import org.mule.weave.v2.model.capabilities.UnknownLocationCapable
import org.mule.weave.v2.model.service.WeaveRuntimePrivilege
import org.mule.weave.v2.model.structure.KeyValuePair
import org.mule.weave.v2.model.types.ArrayType
import org.mule.weave.v2.model.types.ObjectType
import org.mule.weave.v2.model.types.StringType
import org.mule.weave.v2.model.values.ArrayValue
import org.mule.weave.v2.model.values.ObjectValue
import org.mule.weave.v2.model.values.Value
import org.mule.weave.v2.model.values.wrappers.WrapperValue
import org.mule.weave.v2.module.commons.java.value.JavaSchema
import org.mule.weave.v2.module.commons.java.value.JavaValue
import org.mule.weave.v2.module.commons.java.writer.JavaAdapter
import org.mule.weave.v2.module.java.reflection.ReflectionUtils
import org.mule.weave.v2.module.pojo.ClassLoaderService
import org.mule.weave.v2.module.pojo.DefaultClassLoaderService
import org.mule.weave.v2.module.pojo.JavaDataFormat
import org.mule.weave.v2.module.pojo.exception.IllegalReadPropertyAccessException
import org.mule.weave.v2.module.pojo.exception.JavaPropertyAccessException
import org.mule.weave.v2.module.pojo.reader.ReflectionJavaValueConverter
import org.mule.weave.v2.module.writer.Writer
import org.mule.weave.v2.parser.exception.LocatableException

import java.lang.reflect.Field

class JavaFieldRefFunction extends SecureBinaryFunctionValue {

  override val L = StringType

  override val R = StringType

  override val requiredPrivilege: WeaveRuntimePrivilege = WeaveRuntimePrivilege.JAVA_REFLECTION

  override def onSecureExecution(classNameValue: L.V, fieldNameValue: R.V)(implicit ctx: EvaluationContext): Value[_] = {
    val className: String = classNameValue.evaluate.toString
    val fieldName: String = fieldNameValue.evaluate.toString
    try {
      val classToCall = JavaInvocationHelper.loadClass(className)
      val field: Field = classToCall.getDeclaredField(fieldName)
      if (!ctx.serviceManager.settingsService.java().disableSetAccessible) {
        ReflectionUtils.safeTrySetAccessible(field)
      }
      ReflectionJavaValueConverter.convert(field.get(null), () => className + "." + fieldName)
    } catch {
      case le: LocatableException =>
        throw le
      case e @ (_: IllegalAccessException | _: IllegalArgumentException) =>
        throw new IllegalReadPropertyAccessException(fieldName, className, this, e.getCause)
      case e: Exception =>
        throw new JavaPropertyAccessException(fieldName, className, fieldNameValue, e)
    }
  }
}

object JavaFieldRefFunction {

  def transformArgumentsToJava(ctx: EvaluationContext, paramTypes: Seq[Class[_]], argsSeq: Seq[Value[_]]): Seq[AnyRef] = {
    argsSeq
      .zip(paramTypes)
      .map((value) => {
        transformToJava(value._1, value._2)(ctx)
      })
  }

  def transformToJavaMap(arg: Value[_], mapValueClass: Class[_], mapClass: Class[_])(implicit ctx: EvaluationContext): AnyRef = {
    val writer: Writer = JavaDataFormat.defaultSettingsWriter(None)
    val map = ObjectType
      .coerce(arg)
      .evaluate
      .toIterator()
      .map((kvp) => {
        KeyValuePair(kvp._1, adaptToJavaClass(kvp._2, mapValueClass))
      })
    val valueToWrite = adaptToJavaClass(ObjectValue(map, UnknownLocationCapable), mapClass)
    writer.writeValue(valueToWrite)(ctx)
    writer.result.asInstanceOf[AnyRef]
  }

  def transformToJavaCollection(argValue: Value[_], item: Class[_], collectionClass: Class[_])(implicit ctx: EvaluationContext): AnyRef = {
    val writer: Writer = JavaDataFormat.writer(None)
    val values: Iterator[Value[_]] = ArrayType
      .coerce(argValue)
      .evaluate
      .toIterator()
      .map((value) => {
        adaptToJavaClass(value, item)
      })
    val valueToWrite = adaptToJavaClass(ArrayValue(values, UnknownLocationCapable), collectionClass)
    writer.writeValue(valueToWrite)(ctx)
    writer.result.asInstanceOf[AnyRef]
  }

  def transformToJava(argValue: Value[_], expectedJavaType: Class[_])(implicit ctx: EvaluationContext): AnyRef = {
    val maybeAdapter = ValueToJavaAdaptorManager.findAdapterForClass(expectedJavaType)
    maybeAdapter match {
      case Some(valueAdapter) => {
        valueAdapter.adaptToJavaValue(argValue, expectedJavaType, ctx)
      }
      case None => {
        val writer: Writer = JavaDataFormat.defaultSettingsWriter(None)
        val valueToWrite: Value[_] = adaptToJavaClass(argValue, expectedJavaType)
        writer.writeValue(valueToWrite)(ctx)
        JavaAdapter.fromScalaToJava(writer.result).asInstanceOf[AnyRef]
      }
    }
  }

  def adaptToJavaClass(argValue: Value[_], expectedJavaType: Class[_])(implicit ctx: EvaluationContext): Value[_] = {
    //Should we inferred the class?
    val hasClassDefined: Boolean = doesHasClassDefined(argValue, expectedJavaType)
    //If it says object don't use it
    val valueToWrite = if (!hasClassDefined && !classOf[Object].equals(expectedJavaType)) {
      val typeCoercer = argValue.valueType(ctx).baseType.coercer()
      argValue.valueType(ctx).baseType.withSchema(Some(JavaSchema(expectedJavaType))).coerce(argValue)(ctx)
    } else {
      argValue
    }
    valueToWrite
  }

  private def doesHasClassDefined(argValue: Value[_], expectedJavaType: Class[_])(implicit ctx: EvaluationContext): Boolean = {
    argValue.schema match {
      case Some(schema) => {
        argValue match {
          case jv: JavaValue[_] => {
            val underlying = jv.underlying()
            expectedJavaType.isInstance(underlying)
          }
          case wv: WrapperValue => doesHasClassDefined(wv.value, expectedJavaType)
          case _ => {
            val maybeString = schema.`class`
            maybeString match {
              case None => false
              case Some(value) => {
                val classLoaderService = ctx.serviceManager.lookupCustomService(classOf[ClassLoaderService], DefaultClassLoaderService)
                classLoaderService.loadClass(value) match {
                  case None              => false
                  case Some(loadedClass) => expectedJavaType.isAssignableFrom(loadedClass)
                }
              }
            }
          }
        }
      }
      case _ => false
    }
  }
}
